# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
#
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.
# --------------------------------------------------------------------------

from enum import Enum


class JobStatus(str, Enum):

    waiting = "Waiting"
    executing = "Executing"
    succeeded = "Succeeded"
    failed = "Failed"
    cancelled = "Cancelled"


class ProviderAvailability(str, Enum):

    available = "Available"
    degraded = "Degraded"
    unavailable = "Unavailable"


class TargetAvailability(str, Enum):

    available = "Available"
    degraded = "Degraded"
    unavailable = "Unavailable"


class DimensionScope(str, Enum):

    workspace = "Workspace"
    subscription = "Subscription"


class MeterPeriod(str, Enum):

    none = "None"
    monthly = "Monthly"
