# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
#
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.
# --------------------------------------------------------------------------

from msrest.serialization import Model
from msrest.exceptions import HttpOperationError


class RestError(Model):
    """Error information returned by the API.

    :param error:
    :type error: ~azure.quantum.models.ErrorData
    """

    _attribute_map = {
        'error': {'key': 'error', 'type': 'ErrorData'},
    }

    def __init__(self, *, error=None, **kwargs) -> None:
        super(RestError, self).__init__(**kwargs)
        self.error = error


class RestErrorException(HttpOperationError):
    """Server responsed with exception of type: 'RestError'.

    :param deserialize: A deserializer
    :param response: Server response to be deserialized.
    """

    def __init__(self, deserialize, response, *args):

        super(RestErrorException, self).__init__(deserialize, response, 'RestError', *args)
